@@ -1,6 +1,7 @@
 package App::Alice::CommandDispatch;
 
 use Any::Moose;
+use Encode;
 my $SRVOPT = qr/(?:\-(\S+)\s+)?/;
 
 has 'handlers' => (
@@ -10,7 +11,7 @@ has 'handlers' => (
     my $self = shift;
     [
       {sub => '_say',     re => qr{^([^/].*)}s},
-      {sub => 'msg',      re => qr{^/(?:msg|query)\s+$SRVOPT(\S+)(?:\s+(.+))?}},
+      {sub => 'msg',      re => qr{^/(?:msg|query)\s+$SRVOPT(\S+)(.*)}},
       {sub => 'nick',     re => qr{^/nick\s+(\S+)}},
       {sub => 'names',    re => qr{^/n(?:ames)?}, in_channel => 1},
       {sub => '_join',    re => qr{^/j(?:oin)?\s+$SRVOPT(.+)}},
@@ -45,9 +46,7 @@ sub handle {
     if ($command =~ /$re/) {
       my @args = grep {defined $_} ($5, $4, $3, $2, $1); # up to 5 captures
       if ($handler->{in_channel} and !$window->is_channel) {
-        $self->broadcast(
-          $window->format_announcement("$command can only be used in a channel")
-        );
+        $self->reply($window, "$command can only be used in a channel");
       }
       else {
         my $sub = $handler->{sub};
@@ -55,8 +54,7 @@ sub handle {
           $self->$sub($window, @args);
         }
         else {
-          $self->broadcast($self->app->log_info(
-            "Error handling $command: $sub sub not found"));
+          $self->reply($window, "Error handling $command: $sub sub not found");
         }
       }
       return;
@@ -66,24 +64,37 @@ sub handle {
 
 sub names {
   my ($self, $window) = @_;
-  $self->broadcast($window->format_announcement($window->nick_table));
+  $self->reply($window, $window->nick_table);
 }
 
 sub whois {
   my ($self, $window, $nick) = @_;
-  $self->broadcast($window->format_announcement($window->irc->whois_table($nick)));
+  $self->reply($window, $window->irc->whois_table($nick));
 }
 
 sub msg {
-  my ($self, $window, $msg, $nick, $network) = @_;
+  my $self = shift;
+  my $window = shift;
+  my ($msg, $nick, $network);
+  if (@_ == 3) {
+    ($msg, $nick, $network) = @_;
+  }
+  elsif (@_ == 2) {
+    ($msg, $nick) = @_;
+  }
+  $msg =~ s/^\s+//;
   my $irc = $window->irc;
   if ($network and $self->app->has_irc($network)) {
     $irc = $self->app->get_irc($network);
   }
   return unless $irc;
   my $new_window = $self->app->find_or_create_window($nick, $irc);
-  $self->broadcast($new_window->join_action);
-  $irc->cl->send_srv(PRIVMSG => $nick, $msg);
+  my @msgs = ($new_window->join_action);
+  if ($msg) {
+    push @msgs, $new_window->format_message($new_window->nick, $msg);
+    $irc->send_srv(PRIVMSG => $nick, $msg) if $msg;
+  }
+  $self->broadcast(@msgs);
 }
 
 sub _join {
@@ -94,8 +105,8 @@ sub _join {
   }
   my @params = split /\s+/, $channel;
   if ($irc and $irc->cl->is_channel_name($params[0])) {
-    $self->broadcast($irc->log_info("joining $params[0]"));
-    $irc->cl->send_srv(JOIN => @params);
+    $irc->log(info => "joining $params[0]");
+    $irc->send_srv(JOIN => @params);
   }
 }
 
@@ -106,13 +117,13 @@ sub part {
 
 sub close {
   my ($self, $window) = @_;
-  $window->is_channel ?
-    $window->part : $self->app->close_window($window);
+  $window->is_channel ? $window->irc->send_srv(PART => $window->title)
+                      : $self->app->close_window($window);
 }
 
 sub nick {
   my ($self, $window, $nick) = @_;
-  $window->irc->cl->send_srv(NICK => $nick);
+  $window->irc->send_srv(NICK => $nick);
 }
 
 sub create {
@@ -130,7 +141,8 @@ sub clear {
 sub topic {
   my ($self, $window, $new_topic) = @_;
   if ($new_topic) {
-    $window->set_topic($new_topic);
+    $window->topic({string => $new_topic, nick => $window->nick, time => time});
+    $window->irc->send_srv(TOPIC => $window->title, $new_topic);
   }
   else {
     my $topic = $window->topic;
@@ -140,13 +152,13 @@ sub topic {
 
 sub me {
   my ($self, $window, $action) = @_;
-  $self->broadcast($window->format_message($window->nick, "• $action"));
-  $window->irc->cl->send_srv(PRIVMSG => $window->title, chr(01) . "ACTION $action" . chr(01));
+  $self->show($window, "• $action");
+  $window->irc->send_srv(PRIVMSG => $window->title, chr(01) . "ACTION $action" . chr(01));
 }
 
 sub quote {
   my ($self, $window, $command) = @_;
-  $window->irc->cl->send_raw($command);
+  $window->irc->send_raw($command);
 }
 
 sub disconnect {
@@ -155,6 +167,13 @@ sub disconnect {
   if ($irc and $irc->is_connected) {
     $irc->disconnect;
   }
+  elsif ($irc->reconnect_timer) {
+    $irc->cancel_reconnect;
+    $irc->log(info => "canceled reconnect");
+  }
+  else {
+    $self->reply($window, "already disconnected");
+  }
 }
 
 sub connect {
@@ -169,31 +188,41 @@ sub ignores {
   my ($self, $window) = @_;
   my $msg = join ", ", $self->app->ignores;
   $msg = "none" unless $msg;
-  $self->broadcast($window->format_announcement("Ignoring:\n$msg"));
+  $self->reply($window, "Ignoring:\n$msg");
 }
 
 sub ignore {
   my ($self, $window, $nick) = @_;
   $self->app->add_ignore($nick);
-  $self->broadcast($window->format_announcement("Ignoring $nick"));
+  $self->reply($window, "Ignoring $nick");
 }
 
 sub unignore {
   my ($self, $window, $nick) = @_;
   $self->app->remove_ignore($nick);
-  $self->broadcast($window->format_announcement("No longer ignoring $nick"));
+  $self->reply($window, "No longer ignoring $nick");
 }
 
 sub notfound {
   my ($self, $window, $command) = @_;
-  $self->broadcast($window->format_announcement("Invalid command $command"));
+  $self->reply($window, "Invalid command $command");
 }
 
 sub _say {
   my ($self, $window, $msg) = @_;
-  $self->app->logger->log_message(time, $window->nick, $window->title, $msg);
-  $self->broadcast($window->format_message($window->nick, $msg));
-  $window->irc->cl->send_srv(PRIVMSG => $window->title, $msg);
+  $self->app->store($window->nick, $window->title, $msg);
+  $self->show($window, $msg);
+  $window->irc->send_srv(PRIVMSG => $window->title, $msg);
+}
+
+sub show {
+  my ($self, $window, $message) = @_;
+  $self->broadcast($window->format_message($window->nick, $message));
+}
+
+sub reply {
+  my ($self, $window, $message) = @_;
+  $self->broadcast($window->format_announcement($message));
 }
 
 sub broadcast {
